function love.graphics.hexagon(mode, x,y,l)
    local i=(l/2)*3^0.5
    love.graphics.polygon(mode, x,y,x+l,y,x+1.5*l,y+i,x+l,y+2*i,x,y+2*i,x-l*0.5,y+i)
end


function math.getDistance(x1,y1,x2,y2)
	return ((x1-x2)^2+(y1-y2)^2)^0.5
end


function math.axisRot(x,y,rot)
	local xx=math.cos(rot)*x-math.sin(rot)*y
	local yy=math.cos(rot)*y+math.sin(rot)*x
	return xx,yy
end

function math.sign(x)
	if x>0 then return 1
	elseif x<0 then return -1
	else return 0 end
end

function math.getRot(x1,y1,x2,y2,toggle)
	if x1==x2 and y1==y2 then return 0 end 
	local angle=math.atan((x1-x2)/(y1-y2))
	if y1-y2<0 then angle=angle-math.pi end
	if toggle==true then angle=angle+math.pi end
	if angle>0 then angle=angle-2*math.pi end
	if angle==0 then return 0 end
	return -angle
end

function love.graphics.rect(x,y,w,h)
	local r,g,b = love.graphics.getColor()
	love.graphics.setColor(r,g,b,100)
	love.graphics.rectangle('fill', x,y,w,h)
	love.graphics.setColor(r,g,b)
	love.graphics.rectangle('line', x,y,w,h)
end

function love.graphics.polygon2(...)
	local r,g,b,a = love.graphics.getColor()
	love.graphics.setColor(r/2,g/2,b/2,a)
	love.graphics.polygon('fill', ...)
	love.graphics.setColor(r,g,b,a)
	love.graphics.polygon('line', ...)
end


function love.graphics.randomColor()
	local r=math.random(0, 255)
	local g=math.random(0,255)
	local b=math.random(0,255)
	return {r,g,b,255}
end

function math.clamp(a,low,high)
	if a>math.pi then a=a-2*math.pi; end
	return math.max(low,math.min(a,high))
end

function love.graphics.polygon3(mode,x,y,rot,v)
	local tab={}
	for i=1,#v/2 do
		tab[2*i-1],tab[2*i]=math.axisRot(v[2*i-1],v[2*i],rot)
		tab[2*i-1]=tab[2*i-1]+x
		tab[2*i]=tab[2*i]+y
		love.graphics.circle("fill", tab[2*i-1], tab[2*i],3)
	end
	love.graphics.polygon(mode, tab)
end

function math.polygonTrans(x,y,rot,size,v)
	local tab={}
	for i=1,#v/2 do
		tab[2*i-1],tab[2*i]=math.axisRot(v[2*i-1],v[2*i],rot)
		tab[2*i-1]=tab[2*i-1]*size+x
		tab[2*i]=tab[2*i]*size+y
		love.graphics.circle("fill", tab[2*i-1], tab[2*i],3)
	end
	return tab
end


function math.randomPolygon(count,size)
	local v={}
	local rt={}
	local lastK=0
	local lastX=0
	local lastY=0
	local lastRad=0
	local v_c=count
	for i=1,v_c do
		v[i]={}
		v[i].x=love.math.random(-50,50)*size
		v[i].y=love.math.random(-50,50)*size
	end
	local maxY=-50*size
	local oK=0
	for k,v in pairs(v) do
		if v.y>maxY then
			maxY=v.y
			oK=k
		end	
	end
	lastK=oK
	lastX=v[lastK].x
	lastY=v[lastK].y
	table.insert(rt,v[lastK].x)
	table.insert(rt,v[lastK].y)
	local i=0
	while i<100 do
		i=i+1
		local minRad=2*math.pi
		local minK=0
		for k,v in pairs(v) do
			local rad=math.getRot(lastX,lastY,v.x,v.y,true)
			if rad and rad>lastRad then
				if rad<minRad then
					minRad=rad
					minK=k
				end
			end
		end
		if minK==maxK or minK==0 then return rt end
		lastK=minK
		lastRad=minRad
		lastX=v[lastK].x
		lastY=v[lastK].y
		table.insert(rt,v[lastK].x)
		table.insert(rt,v[lastK].y)
	end
end


function math.toBezier(v,len)
	local tab={}
	local points
	--把曲线闭合
	local vx=v[1]
	local vy=v[2]

	for i=1,#v/2 do
		local c
		if v[2*i+1]==nil then 
			local rot=math.getRot(v[2*i-1],v[2*i],vx,vy)+math.pi
			local cx,cy=(v[2*i-1]+vx)/2,(v[2*i]+vy)/2
			local param= love.math.random(-len,len)
			local dx,dy= math.axisRot(param,0,rot)
			c=love.math.newBezierCurve(v[2*i-1],v[2*i],cx+dx,cy+dy,vx,vy)
		else
			local rot=math.getRot(v[2*i-1],v[2*i],v[2*i+1],v[2*i+2])+math.pi
			local cx,cy=(v[2*i-1]+v[2*i+1])/2,(v[2*i]+v[2*i+2])/2
			local param= love.math.random(-len,len)
			local dx,dy= math.axisRot(param,0,rot)
			c=love.math.newBezierCurve(v[2*i-1],v[2*i],cx+dx,cy+dy,v[2*i+1],v[2*i+2])
		end

		for i=0,1,0.03 do
			local x,y=c:evaluate(i)
			table.insert(tab,x)
			table.insert(tab,y)
		end
	end
	return tab
end

function love.physics.insertMask(fixture,c)
	local tab={fixture:getMask()}
	table.insert(tab,c)
	fixture:setMask(unpack(tab))
end

function love.physics.removeMask(fixture,c)
	local tab={fixture:getMask()}
	for k,v in pairs(tab) do
		if v==c then
			table.remove(tab, k)
			break
		end
	end
	fixture:setMask(tab)
end

function math.round(num, n)
	if n > 0 then
		local scale = math.pow(10, n-1)
		return math.floor(num / scale + 0.5) * scale
	elseif n < 0 then
		local scale = math.pow(10, n)
		return math.floor(num / scale + 0.5) * scale
	elseif n == 0 then
		return num
	end
end
