local car=class("car")
local maxForwardSpeed = 800;
local maxBackwardSpeed = -300;
local backTireMaxDriveForce = 20;
local frontTireMaxDriveForce = backTireMaxDriveForce ;
local backTireMaxLateralImpulse = 1;
local frontTireMaxLateralImpulse =backTireMaxLateralImpulse;


function car:setPosition(px,py)
	self.body:setPosition(px,py)
    for k,v in pairs(self.tires) do
        v.body:setPosition(px,py)     
    end
end


function car:initialize(f,b,p,l)
	self.maxForwardSpeed = 800*f;
	self.maxBackwardSpeed = -300*b;
	self.backTireMaxDriveForce = 20*p;
	self.frontTireMaxDriveForce = backTireMaxDriveForce ;
	self.backTireMaxLateralImpulse = 1*l;
	self.frontTireMaxLateralImpulse =backTireMaxLateralImpulse;



	game.carIndex=game.carIndex+1
	self.index=game.carIndex
	self.type="car"
	self.body = love.physics.newBody(game.world, 0, 0, "dynamic")
	self.body:setAngularDamping(10)
	
	self.posInRoad=1
	self.oPos=1
	self.isAICtrl=true
	self.cmd={}
	local tab={1.5,0,
			3,2.5,
			2.8,5.5,
			1,10,
			-1,10,
			-2.8,5.5,
			-3,2.5,
			-1.5,0}
	for k,v in pairs(tab) do
		tab[k]=v*4
	end
	self.shape = love.physics.newPolygonShape(tab)
	self.fixture = love.physics.newFixture(self.body, self.shape, 0.01)
	self.fixture:setUserData(self)
	self.fixture:setGroupIndex( -1-self.index )

	self.shell={}
	self.shell.shape = love.physics.newPolygonShape(-16,-3,16,-3,16,40,-16,40)
	self.shell.fixture = love.physics.newFixture(self.body, self.shell.shape, 0.01)
	self.shell.fixture:setUserData(self)
	self.shell.fixture:setGroupIndex( -1-self.index )

	self.tires={}
	self.puffs={}
	local jointDef={}
	jointDef.bodyA=self.body
	jointDef.enableLimit=true
    local makeTire=require("objs/tires")
    local tire=makeTire:new(-3*4,0.75*4,self)
    tire:setPara(self.maxForwardSpeed, self.maxBackwardSpeed, self.backTireMaxDriveForce, self.backTireMaxLateralImpulse)
    jointDef.bodyB=tire.body
    local joint = love.physics.newWeldJoint(jointDef.bodyA, jointDef.bodyB, 0, 0, false)
    self.tires[1]=tire



    local makeTire=require("objs/tires")
    local tire=makeTire:new(3*4,0.75*4,self)
    tire:setPara(self.maxForwardSpeed, self.maxBackwardSpeed, self.backTireMaxDriveForce, self.backTireMaxLateralImpulse)
    jointDef.bodyB=tire.body
    local joint = love.physics.newWeldJoint(jointDef.bodyA, jointDef.bodyB, 0, 0, false) 
    self.tires[2]=tire



    local makeTire=require("objs/tires")
    local tire=makeTire:new(-3*4,8.5*4,self)
    tire:setPara(self.maxForwardSpeed, self.maxBackwardSpeed, self.frontTireMaxDriveForce, self.frontTireMaxLateralImpulse)
    jointDef.bodyB=tire.body
    local joint=love.physics.newRevoluteJoint(jointDef.bodyA, jointDef.bodyB, -3*4,8.5*4, false) 
    self.tires[3]=tire
    self.flJoint=joint
    tire.parent=self
    tire.isFront=true


    local makeTire=require("objs/tires")
    local tire=makeTire:new(3*4,8.5*4,self)
    tire:setPara(self.maxForwardSpeed, self.maxBackwardSpeed, self.frontTireMaxDriveForce, self.frontTireMaxLateralImpulse)
    jointDef.bodyB=tire.body
    local joint=love.physics.newRevoluteJoint(jointDef.bodyA, jointDef.bodyB, 3*4,8.5*4, false) 
    self.tires[4]=tire
    self.frJoint=joint
    tire.parent=self
    tire.isFront=true


end

local function clamp(a,low,high)
	if a>math.pi then a=a-2*math.pi end
	return math.max(low,math.min(a,high))
end


function car:addPuff()
	local x= (self.tires[1].body:getX() + self.tires[2].body:getX())/2
	local y= (self.tires[1].body:getY() + self.tires[2].body:getY())/2
	if love.math.random()>0.7 then
		local puff=require("objs/puff"):new(x,y)
		table.insert(self.puffs, puff)
	end
end

function car:puffUpdate()
	for k,v in pairs(self.puffs) do
		v:update()
		if v.destroy==true then
			table.remove(self.puffs, k)
		end
	end
end


function car:puffDraw()
	love.graphics.setColor(255, 255, 255, 30)
	for k,v in pairs(self.puffs) do
		v:draw()
	end
end





function car:update(dt)
	self:ai()

	for i,v in ipairs(self.tires) do
		v:update(dt,self.isAICtrl)
	end
	
	self:addPuff(dt)

	self:puffUpdate(dt)

	local cRot=self.body:getAngle()
	local rRot=road.blocks[self.posInRoad].body:getAngle()
	local dist=math.abs(cRot-rRot)
	local dist2=math.pi*2-dist
	if dist>dist2 then dist=dist2 end

	if self.oPos>self.posInRoad and self.oPos~=#road.blocks and dist>math.pi/2 then
		self.wrongWay=true
	elseif self.oPos<self.posInRoad then
		self.wrongWay=false
	end
	self.oPos=self.posInRoad
end


local function getDist(p1,p2)
	local dist=math.abs(p1-p2)
	local dist2=#road.blocks-math.abs(p1-p2)
	if dist>dist2 then dist=dist2 end
	return dist
end


function car:draw()
	
	local alpha=255-3*getDist(game.car.posInRoad,self.posInRoad)
	if alpha<0 then alpha=10 end

	self:puffDraw()
	for i,v in ipairs(self.tires) do
		v:draw()
	end
	love.graphics.setColor(0, 255, 0, alpha)
	love.graphics.polygon2(self.body:getWorldPoints(self.shape:getPoints()))
	love.graphics.setColor(0, 255, 255, alpha)
	--love.graphics.polygon("line",self.body:getWorldPoints(self.shell.shape:getPoints()))
	--love.graphics.line(self.body:getX(),self.body:getY(),road.blocks[self.testPos].body:getPosition())
end


function car:ai()
	self.cmd={}
	local carSpeed=	self.tires[3]:getForwardVelocity():len()
	local testPos=self.posInRoad+math.ceil(carSpeed/maxForwardSpeed)+2
	self.testPos=testPos
	if testPos>#road.blocks then testPos=testPos-#road.blocks end
	if testPos<=0 then testPos=testPos+#road.blocks end
	local testX,testY=road.blocks[testPos].body:getPosition()
	local localX,localY=self.body:getLocalPoint(testX,testY)
	local localAngle=math.getRot(localX,localY,0,0)
	if localAngle>math.pi then localAngle=localAngle-2*math.pi end
	if localAngle<-math.pi/8 then table.insert(self.cmd,"left") end
	if localAngle>math.pi/8 then table.insert(self.cmd,"right") end
	if math.abs(localAngle)<math.pi/6 then table.insert(self.cmd,"forward") end
	if math.abs(localAngle)>math.pi/3 then table.insert(self.cmd,"brake") end
	if carSpeed<=100 then table.insert(self.cmd,"forward") end
end
return car