local road=class("road")
local function getLastInfo(l)
	---获取上一个块的中心坐标和旋转角度，方便定位

	local ox,oy=l.body:getPosition()
	local r=l.body:getAngle()
	local rx,ry,dx,dy,dr
	if l.model=="straight" then
		dx,dy= math.axisRot(0,50,r)
		dr=0
	elseif l.model=="corner_l" then
		rx,ry=math.axisRot(50+150*l.size,0,math.rad(15))
		dx,dy= math.axisRot(rx-50-150*l.size,ry,r)
		
		dr=math.rad(15)
	elseif  l.model=="corner_r" then
		rx,ry=math.axisRot(-50-150*l.size,0,-math.rad(15))
		dx,dy= math.axisRot(rx+50+150*l.size,ry,r)
		dr=math.rad(-15)
	end
	return dx+ox,dy+oy,r+dr,r
end


function road:initialize()
	--类初始化，所有路径块放在blocks里，index实际为块计数
	self.blocks={}
	self.index=0
	self.highLight=0
	--这里意思是70%生成随机长度为10~20块直线，30%生成随机角度-180,180的转弯
	self:addTrack("s",love.math.random(10,20))
	for i=1,10 do
		if love.math.random()>0.5 then
			self:addTrack("s",love.math.random(10,20))
		else
			self:addTrack("c",15*love.math.random(1,12)*math.sign(love.math.random()-0.5))
		end
	end

	self:loop()
	self:mini()
	--[[
	其实应该不难 上面的代码是随机生成一个跑道，目前的跑道允许交叠，因为只有赛车附近的跑道能够碰撞。
	目前只考虑两种块，一种是直线 一种是15度角的块，旋转中心在最边上，具体可以看addCorner函数，
	现在要做的是将初始位置和结束位置通过跑道连接起来。比较麻烦的是因为块的旋转中心不在body中心上，旋转会
	产生位移。

	]]
end

function road:mini()
	local tab={}
	for k,v in pairs(self.blocks) do
		local x,y=v.body:getPosition()
		table.insert(tab,{(x-10000)/50,(y-10000)/50})
	end
	road.miniMap=tab
end


function road:loop()
	--这里要自动添加块 使得首尾相连，目前还有问题。。。
	local lastX,lastY,lastR=getLastInfo(self.blocks[#self.blocks])

	local firstX,firstY=10000,10000
	local n=math.round(math.deg(math.pi/2-lastR)*100/15,2)*15/100

	self:addTrack("c",n)
	local lastX,lastY,lastR=getLastInfo(self.blocks[#self.blocks])
	local lastX,lastY,lastR=getLastInfo(self.blocks[#self.blocks])
	self:addTrack("s",math.abs(lastX-firstX)/50+10)
	self:addTrack("c",90)
	local lastX,lastY,lastR=getLastInfo(self.blocks[#self.blocks])
	if lastY-firstY>0 then
		self:addTrack("s",(lastY-firstY)/50)
	end
	self:addTrack("s",math.abs(lastY-firstY)/50)
	self:addTrack("c",90)
	local lastX,lastY,lastR=getLastInfo(self.blocks[#self.blocks])
	self:addTrack("s",math.abs(lastX-firstX)/50-5)
	self:addTrack("c",90)

	if lastY-firstY<0 then
		self:addTrack("s",math.floor(math.abs(lastY-firstY)/50)-7)
	end
	self:newComplet()
end





function road:addTrack(type,deg)
	--type c为拐角，deg为角度，s为直线，deg为块数（每个50长）
	deg=deg or 0
	size=size or 1.5
	if type=="c" then
		if deg%15~=0 then error("must be a x15") end
		for i=1,math.abs(deg/15) do
			local t=deg>0
			self.last=self:newCorner(t)
		end
	end
	if type=="s" then
		for i=1,deg do
			self.last=self:newBlock()
		end
	end
end


function road:newBlock()
	---新直线块 left right分别为左右两边护栏
	local block={}

	block.model="straight"
	self.index=self.index+1
	block.index=self.index
	local x,y,r
	if self.last then
		x,y,r=getLastInfo(self.last)
		block.size=self.last.size
		block.angle=self.last.angle
	else
		x,y,r=10000,10000,0
		block.size=1.5
		block.angle=0
	end


	block.body = love.physics.newBody(game.world, x, y, "static")
	block.shape = love.physics.newPolygonShape(-150*block.size,0,-150*block.size,50,150*block.size,50,150*block.size,0)
	block.fixture = love.physics.newFixture(block.body, block.shape)	
	block.body:setAngle(r)
	block.fixture:setSensor( true )
	block.fixture:setGroupIndex( -3000 )
	block.type="sensor"
	block.fixture:setUserData(block)

	block.left={}
	block.left.index=self.index
	block.left.body = love.physics.newBody(game.world, x, y, "static")
	block.left.shape = love.physics.newPolygonShape(-50-150*block.size,0,-50-150*block.size,50,-150*block.size,50,-150*block.size,0)
	block.left.fixture = love.physics.newFixture(block.left.body, block.left.shape)
	block.left.body:setAngle(r)
	block.left.fixture:setRestitution(0.4)
	block.left.fixture:setFriction(1)
	block.left.fixture:setGroupIndex( -3000 )
	block.left.type="wall"
	block.left.fixture:setUserData(block.left)

	block.right={}
	block.right.index=self.index
	block.right.body = love.physics.newBody(game.world, x, y, "static")
	block.right.shape = love.physics.newPolygonShape(50+150*block.size,0,50+150*block.size,50,150*block.size,50,150*block.size,0)
	block.right.fixture = love.physics.newFixture(block.right.body, block.right.shape)
	block.right.body:setAngle(r)
	block.right.fixture:setRestitution(0.4)
	block.right.fixture:setFriction(1)
	block.right.fixture:setGroupIndex( -3000 )
	block.right.type="wall"
	block.right.fixture:setUserData(block.right)
	
	table.insert(self.blocks,block)
	return block
end




function road:newCorner(isleft)
	local block={}

	if self.last then
		x,y,r=getLastInfo(self.last)
		block.size=self.last.size
		block.angle=self.last.angle
	else
		self.last={}
		x,y,r=10000,10000,0
		block.size=1.5
		block.angle=0
		self.last.angle=0
	end


	self.index=self.index+1
	block.index=self.index

	

	local mx,my,lx,ly,lx2,ly2,rx,ry,rx2,ry2

--下面代码是生产旋转后的块上部坐标，mx,my为中心，l l2为左边墙，r r2为右边墙

	if isleft then 
		block.model="corner_l"	
		kx,ky = math.axisRot(50+150*block.size,0,math.rad(15))
		mx,my=kx-50-150*block.size,ky

		kx,ky = math.axisRot(50,0,math.rad(15))
		lx,ly=kx-50-150*block.size,ky

		lx2,ly2=-50-150*block.size,0
		
		kx,ky = math.axisRot(300*block.size+50,0,math.rad(15))
		rx,ry=kx-50-150*block.size,ky

		kx,ky = math.axisRot(300*block.size+100,0,math.rad(15))
		rx2,ry2=kx-50-150*block.size,ky
		block.angle=self.last.angle
	else
		block.model="corner_r"
		kx,ky = math.axisRot(-50-150*block.size,0,math.rad(-15))
		mx,my=kx+50+150*block.size,ky

		kx,ky = math.axisRot(-50,0,math.rad(-15))
		rx,ry=kx+50+150*block.size,ky

		rx2,ry2=50+150*block.size,0
		
		kx,ky = math.axisRot(-300*block.size-50,0,math.rad(-15))
		lx,ly=kx+50+150*block.size,ky

		kx,ky = math.axisRot(-300*block.size-100,0,math.rad(-15))
		lx2,ly2=kx+50+150*block.size,ky
		block.angle=self.last.angle
	end

	block.body = love.physics.newBody(game.world, x, y, "static")
	block.shape = love.physics.newPolygonShape(-150*block.size,0,lx,ly,rx,ry,150*block.size,0)
	block.fixture = love.physics.newFixture(block.body, block.shape)
	block.body:setAngle(r)
	block.fixture:setSensor( true )
	block.type="sensor"
	block.fixture:setUserData(block)

	block.left={}
	block.left.index=self.index
	block.left.body = love.physics.newBody(game.world, x, y, "static")
	block.left.shape = love.physics.newPolygonShape(-50-150*block.size,0,lx2,ly2,lx,ly,-150*block.size,0)
	block.left.fixture = love.physics.newFixture(block.left.body, block.left.shape)
	block.left.body:setAngle(r)
	block.left.fixture:setRestitution(0.4)
	block.left.fixture:setFriction(1)
	block.left.fixture:setGroupIndex( -3000 )
	block.left.type="wall"
	block.left.fixture:setUserData(block.left)

	block.right={}
	block.right.index=self.index
	block.right.body = love.physics.newBody(game.world, x, y, "static")
	block.right.shape = love.physics.newPolygonShape(50+150*block.size,0,rx2,ry2,rx,ry,150*block.size,0)
	block.right.fixture = love.physics.newFixture(block.right.body, block.right.shape)
	block.right.body:setAngle(r)
	block.right.fixture:setRestitution(0.4)
	block.right.fixture:setFriction(1)
	block.right.fixture:setGroupIndex( -3000 )
	block.right.fixture:setUserData(block.right)
	block.right.type="wall"
	table.insert(self.blocks,block)
	return block



end

function road:newTrans(size)
	local block={}

	block.model="straight"
	block.size=size
	self.index=self.index+1
	block.index=self.index
	block.angle=self.last.angle

	local x,y,r=getLastInfo(self.last)
	block.body = love.physics.newBody(game.world, x, y, "static")
	block.shape = love.physics.newPolygonShape(-150*self.last.size,0,-150*block.size,50,150*block.size,50,150*self.last.size,0)
	block.fixture = love.physics.newFixture(block.body, block.shape)
	block.body:setAngle(r)
	block.fixture:setSensor( true )
	block.type="sensor"
	block.fixture:setUserData(block)

	block.left={}
	block.left.index=self.index
	block.left.body = love.physics.newBody(game.world, x, y, "static")
	block.left.shape = love.physics.newPolygonShape(-50-150*self.last.size,0,-50-150*block.size,50,-150*block.size,50,-150*self.last.size,0)
	block.left.fixture = love.physics.newFixture(block.left.body, block.left.shape)
	block.left.body:setAngle(r)
	block.left.fixture:setRestitution(0.4)
	block.left.fixture:setFriction(1)
	block.left.fixture:setGroupIndex(-3000)
	block.left.type="wall"
	block.left.fixture:setUserData(block.left)

	block.right={}
	block.right.index=self.index
	block.right.body = love.physics.newBody(game.world, x, y, "static")
	block.right.shape = love.physics.newPolygonShape(50+150*self.last.size,0,50+150*block.size,50,150*block.size,50,150*self.last.size,0)
	block.right.fixture = love.physics.newFixture(block.right.body, block.right.shape)
	block.right.body:setAngle(r)
	block.right.fixture:setRestitution(0.4)
	block.right.fixture:setFriction(1)
	block.right.fixture:setGroupIndex(-3000 )
	block.right.type="wall"
	block.right.fixture:setUserData(block.right)
	table.insert(self.blocks,block)
	return block
end


function road:newComplet()
	---新直线块 left right分别为左右两边护栏
	local block={}

	block.model="complet"
	self.index=self.index+1
	block.index=self.index
	local x,y,r

	x,y,r=getLastInfo(self.last)
	block.size=self.last.size
	block.angle=self.last.angle


	local f=self.blocks[1]
	local tab={f.left.body:getWorldPoints(f.left.shape:getPoints())}
	local lx2,ly2,lx,ly=tab[1],tab[2],tab[7],tab[8]

	local tab={f.right.body:getWorldPoints(f.right.shape:getPoints())}
	local rx2,ry2,rx,ry=tab[1],tab[2],tab[7],tab[8]	

	print(f.left.shape:getPoints())
	print(f.right.shape:getPoints())
	block.body = love.physics.newBody(game.world, x, y, "static")
	lx2,ly2=block.body:getLocalPoint(lx2,ly2)
	lx,ly=block.body:getLocalPoint(lx,ly)
	rx2,ry2=block.body:getLocalPoint(rx2,ry2)
	rx,ry=block.body:getLocalPoint(rx,ry)

	block.shape = love.physics.newPolygonShape(-150*block.size,0,lx,ly,rx,ry,150*block.size,0)
	block.fixture = love.physics.newFixture(block.body, block.shape)	
	block.body:setAngle(r)
	block.fixture:setSensor( true )
	block.fixture:setGroupIndex( -3000 )
	block.type="sensor"
	block.fixture:setUserData(block)

	block.left={}
	block.left.index=self.index
	block.left.body = love.physics.newBody(game.world, x, y, "static")
	block.left.shape = love.physics.newPolygonShape(-50-150*block.size,0,lx2,ly2,lx,ly,-150*block.size,0)
	block.left.fixture = love.physics.newFixture(block.left.body, block.left.shape)
	block.left.body:setAngle(r)
	block.left.fixture:setRestitution(0.4)
	block.left.fixture:setFriction(1)
	block.left.fixture:setGroupIndex( -3000 )
	block.left.type="wall"
	block.left.fixture:setUserData(block.left)

	block.right={}
	block.right.index=self.index
	block.right.body = love.physics.newBody(game.world, x, y, "static")
	block.right.shape = love.physics.newPolygonShape(50+150*block.size,0,rx2,ry2,rx,ry,150*block.size,0)
	block.right.fixture = love.physics.newFixture(block.right.body, block.right.shape)
	block.right.body:setAngle(r)
	block.right.fixture:setRestitution(0.4)
	block.right.fixture:setFriction(1)
	block.right.fixture:setGroupIndex( -3000 )
	block.right.type="wall"
	block.right.fixture:setUserData(block.right)
	
	table.insert(self.blocks,block)
	return block
end

function road:draw()
	for i=1,100 do
		local v
		local p=game.car.posInRoad-50
		if i+p>#self.blocks then 
			v=self.blocks[i+p-#self.blocks]
		elseif i+p<=0 then
			v=self.blocks[#self.blocks+i+p]
		else 
			v=self.blocks[i+p]
		end
		if love.mouse.isDown("t") then print(i+p) end
		if  v then
			local alpha=255-math.abs(10*(i-50))
			if alpha>255 then alpha=255 end
			if alpha<20 then alpha=20 end
			love.graphics.setColor(255, 0, 0, alpha)
			love.graphics.polygon("line", v.left.body:getWorldPoints(v.left.shape:getPoints()))
			love.graphics.polygon("line", v.right.body:getWorldPoints(v.right.shape:getPoints()))
		end
	end
	
	local v=self.blocks[self.highLight]
	if not v then return end
	love.graphics.setColor(0, 255, 0, 10)
	love.graphics.polygon("fill", v.body:getWorldPoints(v.shape:getPoints()))

end

function road:update()



end

return road