tire=class("tire")
local torquePower=50
local killSpeed=1/50
function tire:initialize(x,y,parent)
	self.parent=parent
	self.type="tire"
	self.body = love.physics.newBody(game.world, x,y, "dynamic")
	self.shape = love.physics.newRectangleShape(0.5*8, 1.25*8)
	self.fixture = love.physics.newFixture(self.body, self.shape,3)
	self.fixture:setGroupIndex( -1-self.parent.index )
	self.fixture:setUserData(self.parent)
	self.maxFSpeed=0
	self.maxBSpeed=0
	self.maxDriveForce=0
	self.isFront=false
	self.trace={}

end

function tire:setPara(mfs,mbs,mdf,mli)
	self.maxFSpeed=mfs
	self.maxBSpeed=mbs
	self.maxDriveForce=mdf
	self.maxLImp=mli
end


function tire:getLateralVelocity()
	local axis=vec()
	axis.x , axis.y=self.body:getWorldVector(1, 0)
	local v=vec()
	v.x,v.y=self.body:getLinearVelocity()
	return (axis*v)*axis
end

function tire:getForwardVelocity()
	local axis=vec()
	axis.x , axis.y=self.body:getWorldVector(0, 1)
	local v=vec()
	v.x,v.y=self.body:getLinearVelocity()
	return (axis*v)*axis
end


function tire:addTrace()
	local pos=vec()
	pos.x,pos.y=self.body:getPosition()
	table.insert(self.trace, 1, pos)
	if #self.trace>51 then 
		table.remove(self.trace, 51)
	end
end

function tire:drawTrace()
	for i=1,300 do
		if not self.trace[i] then return end
		love.graphics.setColor(0, 0, 255, 255-255/i)
		love.graphics.circle("fill", self.trace[i].x, self.trace[i].y, 2, 4)
	end

end

function tire:updateFriction(dt)
	--消除横向速度
	local maxLateralImp=self.maxLImp

	if self.brake and (not self.isFront) then
		maxLateralImp=self.maxLImp/1.2
	elseif self.brake then
		maxLateralImp=self.maxLImp*1.2
	end

	local imp=self.body:getMass() * -self:getLateralVelocity()
	if imp:len()>maxLateralImp then
		if imp:len()>maxLateralImp then
			self:addTrace()
		end
		imp=imp*(maxLateralImp / imp:len())	
	end
	self.body:applyLinearImpulse(imp.x,imp.y)
	--消除角速度
	self.body:applyAngularImpulse(  0.6* self.body:getInertia()* -self.body:getAngularVelocity())
	--self.body:setAngularVelocity(0)
	--前进线速度
	if self.brake then
		--[[
		local imp=self.body:getMass() * -self:getForwardVelocity()
		if imp:len()>maxLateralImp then
			imp=imp*(maxLateralImp / imp:len())
		end
		self.body:applyLinearImpulse(imp.x,imp.y)]]
	else
		local v=self:getForwardVelocity()
		local currentSpeed=v:len()
		local currentNormal=v:normalized()
		local drag=-killSpeed*currentSpeed
		local f=drag*currentNormal
		self.body:applyForce(f.x,f.y)
	end
end


function tire:updateDrive(dt,ai)
	--if self.brake then return end
	
	local ds=0
	self.brake=false
	if ai then
		for k,v in pairs(self.parent.cmd) do
			if v=="forward" then ds=self.maxFSpeed end
			if v=="back" then ds=self.maxBSpeed end
			if v=="brake" then self.brake=true end
		end
	else
		if love.keyboard.isDown("w") then
			ds=self.maxFSpeed
		end
		if love.keyboard.isDown("s") then
			ds=self.maxBSpeed
		end
		if love.keyboard.isDown(" ") then
			self.brake=true
		end
	end

	if ds==0 then return end
	---获取当前速度
	local axis=vec()
	axis.x , axis.y=self.body:getWorldVector(0, 1)
	local v=vec()
	v.x,v.y=self.body:getLinearVelocity()
	local cs=v*axis
	---施加力
	local f=0
	if ds>cs then
		f=self.maxDriveForce
	elseif ds<cs then
		f=-self.maxDriveForce
	else
		return
	end
	self.body:applyForce(f*axis.x,f*axis.y)

end

function tire:updateTurn(dt,ai)
	if not self.isFront then return end
	local torque=torquePower
	local dtorque=0 
	if ai then
		for k,v in pairs(self.parent.cmd) do
			if v=="left" then dtorque=-torque end
			if v=="right" then dtorque=torque end
		end
	else
		if love.keyboard.isDown("a") then
			dtorque=-torque
		end
		if love.keyboard.isDown("d") then
			dtorque=torque
		end
	end

	local cAngle=self.parent.body:getAngle()
	local tAngle=self.body:getAngle()
	local dAngle=cAngle-tAngle
	
	local rate=(self.maxFSpeed-self:getForwardVelocity():len())/self.maxFSpeed
	
	local maxAngle
	if self.brake then 
		maxAngle=math.rad(30)
	else
		maxAngle=math.rad(2+30*rate)
	end

	if dAngle>maxAngle then
		self.body:setAngle(cAngle-maxAngle)
	elseif dAngle<-maxAngle then
		self.body:setAngle(cAngle+maxAngle)

	else
		self.body:applyTorque(dtorque)
	end
	if dAngle>0.1 and dtorque==0 then
		self.body:applyTorque(torque)
	elseif dAngle<-0.1 and dtorque==0 then
		self.body:applyTorque(-torque)
	end
end

function tire:draw()
	self:drawTrace()
	love.graphics.setColor(255, 0, 0, 255)
	love.graphics.polygon2(self.body:getWorldPoints(self.shape:getPoints()))
end

function tire:update(dt,toggle)
	self:updateFriction(dt)
	self:updateDrive(dt,toggle)
	self:updateTurn(dt,toggle)
end

return tire
